from django.db import models


class ModelType(models.Model):
    name = models.CharField(max_length=100, verbose_name='名称')

    def __str__(self):
        return self.name

    class Meta:
        verbose_name = '选项类型'
        verbose_name_plural = verbose_name


class ModelX(models.Model):
    model_filed1 = models.CharField(max_length=100, verbose_name="模型X字段1")
    model_filed2 = models.CharField(max_length=100, verbose_name="模型X字段2")
    model_filed3 = models.CharField(max_length=100, verbose_name="模型X字段3")
    model_filed4 = models.CharField(max_length=100, verbose_name="模型X字段4")
    model_filed5 = models.CharField(max_length=100, verbose_name="模型X字段5")
    model_filed6 = models.CharField(max_length=100, verbose_name="模型X字段6")
    model_filed7 = models.ForeignKey(ModelType, on_delete=models.CASCADE, blank=True, null=True,
                                          verbose_name="模型X字段7")

    def __str__(self):
        return self.model_filed1


class ModelA(models.Model):
    name = models.CharField(max_length=100)
    description = models.TextField()
    modelx = models.ManyToManyField(ModelX, related_name='model_as')

    def __str__(self):
        return self.name


class ModelB(models.Model):
    model_a = models.ForeignKey(ModelA, on_delete=models.CASCADE, related_name='model_bs')
    title = models.CharField(max_length=100)
    content = models.TextField()

    def __str__(self):
        return self.title


class ModelC(models.Model):
    model_a = models.ForeignKey(ModelA, on_delete=models.CASCADE, related_name='model_cs')
    title = models.CharField(max_length=100)
    content = models.TextField()

    def __str__(self):
        return self.title



from django.contrib import admin

from pdfExport.admin import PdfAdmin
from .models import ModelA, ModelB, ModelType, ModelX, ModelC


# Register your models here.
@admin.register(ModelType)
class ModelTypeAdmin(PdfAdmin):
    list_display = ('name',)


@admin.register(ModelX)
class ModelXAdmin(PdfAdmin):
    list_display = ('model_filed1', 'model_filed2', 'model_filed3', 'pdf_show')
    pdf_fields = (('model_filed1', 'model_filed2'),
                  ('组标题1', {('model_filed3', 'model_filed4')}),
                  'model_filed5',
                  ('组标题2', {('model_filed6',)}),
                  'model_filed7')
    option_fields = ('model_filed7',)


class ModelBInline(admin.TabularInline):
    model = ModelB
    extra = 1  # 默认显示的空白关联对象数量

class ModelCInline(admin.TabularInline):
    model = ModelC
    extra = 1  # 默认显示的空白关联对象数量


class ModelAAdmin(PdfAdmin):
    list_display = ('name', 'description', 'pdf_show')  # 在Admin列表页显示的字段
    inlines = [ModelBInline, ModelCInline]  # 在ModelA的编辑页面嵌入ModelB的编辑表单
    # pdf要显示的字段
    pdf_related_fields = [

        ('modelx', '自定义标题x', (('model_filed1', 'model_filed2'),
                  ('组标题1', {('model_filed3', 'model_filed4')}),
                  'model_filed5',
                  ('组标题2', {('model_filed6',)}),
                  'model_filed7')),
        ('model_bs', '自定义标题b', ('title', 'content')),
        ('model_cs', '自定义标题c', ('title', 'content')),
    ]
    option_fields = ('model_filed7',)

    # pdf_related_fields = ('model_bs', 'model_cs')
    pdf_fields = (('name', 'description'),)
    # pdf标题
    pdf_title = 'pdf标题'
    # 要显示所有选项的字段
    # 左右上方小标题
    left_tip = '编号AAA'
    right_tip = '编号BBB'


# 注册ModelA到admin中，并使用ModelAAdmin进行定制
admin.site.register(ModelA, ModelAAdmin)


# 如果你还希望单独管理ModelB，也可以注册ModelB
class ModelBAdmin(PdfAdmin):
    list_display = ('title', 'content', 'model_a')  # 在Admin列表页显示的字段
    list_filter = ('model_a',)  # 列表页过滤器

class ModelCAdmin(PdfAdmin):
    list_display = ('title', 'content', 'model_a')  # 在Admin列表页显示的字段
    list_filter = ('model_a',)  # 列表页过滤器


admin.site.register(ModelB, ModelBAdmin)
admin.site.register(ModelC, ModelCAdmin)



# Generated by Django 5.0.2 on 2024-03-18 08:19

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example_pdf', '0005_modela_modelx'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='modelx',
            name='model_filed7',
        ),
        migrations.AddField(
            model_name='modelx',
            name='model_filed7',
            field=models.ManyToManyField(blank=True, null=True, to='example_pdf.modeltype', verbose_name='模型X字段7'),
        ),
    ]



# Generated by Django 5.0.2 on 2024-03-18 08:17

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example_pdf', '0004_modeltype_modelx'),
    ]

    operations = [
        migrations.AddField(
            model_name='modela',
            name='modelx',
            field=models.ForeignKey(default=1, on_delete=django.db.models.deletion.CASCADE, related_name='model_as', to='example_pdf.modelx'),
            preserve_default=False,
        ),
    ]



# Generated by Django 5.0.2 on 2024-03-18 03:05

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example_pdf', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='ModelA',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100)),
                ('description', models.TextField()),
            ],
        ),
        migrations.CreateModel(
            name='ModelB',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=100)),
                ('content', models.TextField()),
                ('model_a', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='model_bs', to='example_pdf.modela')),
            ],
        ),
        migrations.DeleteModel(
            name='TodoItem',
        ),
        migrations.DeleteModel(
            name='TodoType',
        ),
    ]



from django.apps import apps
from django.contrib import admin, messages
from django.urls import path, reverse
from django.http import HttpResponse
from django.template.loader import render_to_string
from django.utils.html import format_html

from django.core.exceptions import FieldDoesNotExist
from django.contrib import admin
from django.urls import path
from django.http import HttpResponse
from django.template.loader import render_to_string
from django.shortcuts import render

from django.contrib import admin
from django.urls import path
from django.http import HttpResponse
from django.template.loader import render_to_string
from django.shortcuts import render


class OptionsShow:
    """
    一个用于标记字段需要特殊显示逻辑的标志类。
    """
    pass


class PdfAdmin(admin.ModelAdmin):
    pdf_fields = []
    option_fields = []
    pdf_related_fields = []
    pdf_title = ''
    right_tip = ''
    left_tip = ''
    pdf_template = 'pdfExport/base_pdf_template.html'
    change_form_template = 'pdfExport/pdf_export_change_form.html'

    def get_urls(self):
        urls = super().get_urls()
        custom_urls = [
            path('pdf_export/<int:pk>/', self.admin_site.admin_view(self.pdf_export), name='pdf_export'),
        ]
        return custom_urls + urls

    def pdf_show(self, obj):
        # 获取当前模型的 app_label 和 model_name
        app_label = obj._meta.app_label
        model_name = obj._meta.model_name

        # 构建 URL
        url = f'/admin/{app_label}/{model_name}/pdf_export/{obj.pk}/'

        return format_html('<a class="button" href="{}">PDF导出</a>', url)

    pdf_show.short_description = 'PDF导出'
    pdf_show.allow_tags = True

    def pdf_export(self, request, pk):
        context = self.get_pdf_context(pk)
        context['title'] = self.pdf_title
        context['right_tip'] = self.right_tip
        context['left_tip'] = self.left_tip
        return render(request, self.pdf_template, context)



    def get_pdf_context(self, pk):
        model = self.model
        app_label = model._meta.app_label
        model_name = model._meta.model_name
        context = {}
        context['grouped_fields'] = self.extract_pdf_data(app_label, model_name, pk)
        if self.pdf_related_fields:
            context['related_grouped_fields'] = self.extract_pdf_related_data(app_label, model_name, pk)
        return context

    def extract_pdf_related_data(self, app_label, model_name, pk):
        model = apps.get_model(app_label, model_name)
        instance = model.objects.get(pk=pk)

        related_data = {}
        for field_name, custom_title, fields_structure in self.pdf_related_fields:
            related_objects = getattr(instance, field_name).all()
            data_list = []
            for obj in related_objects:
                data_list.append(self.process_fields_structure_for_related(obj, fields_structure))
            related_data[custom_title] = data_list

        return related_data

    def process_fields_structure_for_related(self, obj, fields_structure):
        group_data = []
        option_fields = getattr(self, 'option_fields', [])
        for item in fields_structure:
            if isinstance(item, tuple):
                # 处理字段组或单个字段组合
                if isinstance(item[1], set):
                    # 处理具有自定义组标题的字段组
                    group_title, fields_set = item
                    field_group = {"group_title": group_title, "fields": []}
                    for sub_field_tuple in fields_set:
                        for sub_field in sub_field_tuple:
                            field_data = self.process_field(obj._meta.get_field(sub_field), obj, sub_field, option_fields)
                            field_group["fields"].append(field_data)
                    group_data.append(field_group)
                else:
                    # 处理没有自定义组标题的字段组合
                    field_group = {"group_title": None, "fields": []}
                    for field_name in item:
                        field_data = self.process_field(obj._meta.get_field(field_name), obj, field_name, option_fields)
                        field_group["fields"].append(field_data)
                    group_data.append(field_group)
            else:
                # 处理单个字段
                field_data = self.process_field(obj._meta.get_field(item), obj, item, option_fields)
                group_data.append({"group_title": None, "fields": [field_data]})
        return group_data

    def extract_pdf_data(self, app_label, model_name, pk):
        model = apps.get_model(app_label=app_label, model_name=model_name)
        obj = model.objects.get(pk=pk)
        grouped_fields = []
        option_fields = getattr(self, 'option_fields', [])

        for item in self.pdf_fields:
            if isinstance(item, tuple):
                if isinstance(item[1], set):
                    group_title, fields_set = item
                    field_group = {"group_title": group_title, "fields": []}
                    for sub_field_tuple in fields_set:
                        for sub_field in sub_field_tuple:
                            field_obj = model._meta.get_field(sub_field)
                            field_data = self.process_field(field_obj, obj, sub_field, option_fields)
                            field_group["fields"].append(field_data)
                    grouped_fields.append(field_group)
                else:
                    field_group = {"group_title": None, "fields": []}
                    for sub_field in item:
                        field_obj = model._meta.get_field(sub_field)
                        field_data = self.process_field(field_obj, obj, sub_field, option_fields)
                        field_group["fields"].append(field_data)
                    grouped_fields.append(field_group)
            else:
                field_obj = model._meta.get_field(item)
                field_data = self.process_field(field_obj, obj, item, option_fields)
                grouped_fields.append({"group_title": None, "fields": [field_data]})

        return grouped_fields

    def process_field(self, field_obj, obj, field_name, option_fields):
        """处理单个字段，获取其详细信息"""
        if field_name in option_fields:
            # 选项字段处理逻辑
            all_options = field_obj.related_model.objects.all()
            current_option = getattr(obj, field_name, None)
            options_list = [{'name': str(option), 'selected': (option == current_option)} for option in all_options]
            return {
                "verbose_name": field_obj.verbose_name,
                "value": options_list,
                "is_option_field": True
            }
        else:
            # 非选项字段处理逻辑
            return {
                "verbose_name": field_obj.verbose_name,
                "value": getattr(obj, field_name, ''),
                "is_option_field": False
            }



# Generated by Django 5.0.2 on 2024-03-18 03:44

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('example_pdf', '0003_modelc'),
    ]

    operations = [
        migrations.CreateModel(
            name='ModelType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='名称')),
            ],
            options={
                'verbose_name': '选项类型',
                'verbose_name_plural': '选项类型',
            },
        ),
        migrations.CreateModel(
            name='ModelX',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('model_filed1', models.CharField(max_length=100, verbose_name='模型X字段1')),
                ('model_filed2', models.CharField(max_length=100, verbose_name='模型X字段2')),
                ('model_filed3', models.CharField(max_length=100, verbose_name='模型X字段3')),
                ('model_filed4', models.CharField(max_length=100, verbose_name='模型X字段4')),
                ('model_filed5', models.CharField(max_length=100, verbose_name='模型X字段5')),
                ('model_filed6', models.CharField(max_length=100, verbose_name='模型X字段6')),
                ('model_filed7', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='example_pdf.modeltype', verbose_name='模型X字段7')),
            ],
        ),
    ]



# Generated by Django 5.0.2 on 2024-03-07 02:34

import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='TodoType',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=100, verbose_name='名称')),
            ],
            options={
                'verbose_name': '待办类型',
                'verbose_name_plural': '待办类型',
            },
        ),
        migrations.CreateModel(
            name='TodoItem',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('title', models.CharField(max_length=255, verbose_name='标题')),
                ('subtitle', models.CharField(blank=True, max_length=255, null=True, verbose_name='副标题')),
                ('sys_name', models.CharField(default='', max_length=100, verbose_name='系统名称')),
                ('todo_uid', models.CharField(max_length=100, unique=True, verbose_name='待办UID')),
                ('redirect_url', models.URLField(verbose_name='待办跳转URL')),
                ('assignee', models.CharField(max_length=255, verbose_name='待办人')),
                ('assignee_id', models.CharField(max_length=100, verbose_name='待办人ID')),
                ('initiator', models.CharField(blank=True, max_length=255, null=True, verbose_name='发起人')),
                ('initiator_id', models.CharField(blank=True, max_length=100, null=True, verbose_name='发起人ID')),
                ('handler', models.CharField(blank=True, max_length=255, null=True, verbose_name='经办人')),
                ('handler_id', models.CharField(blank=True, max_length=100, null=True, verbose_name='经办人ID')),
                ('reminder_count', models.IntegerField(default=0, verbose_name='催办次数')),
                ('notes', models.TextField(blank=True, null=True, verbose_name='备注')),
                ('is_completed', models.BooleanField(default=False, verbose_name='已完成状态')),
                ('completion_time', models.DateTimeField(blank=True, null=True, verbose_name='办结时间')),
                ('completer', models.CharField(blank=True, max_length=255, null=True, verbose_name='办结人')),
                ('completer_id', models.CharField(blank=True, max_length=100, null=True, verbose_name='办结人ID')),
                ('created_at', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='更新时间')),
                ('due_date', models.DateField(blank=True, null=True, verbose_name='截止日期')),
                ('type', models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='example_pdf.todotype', verbose_name='类型')),
            ],
            options={
                'verbose_name': '待办事项',
                'verbose_name_plural': '待办事项',
            },
        ),
    ]



from setuptools import setup, find_packages

setup(
    name='qz_pdf_export',
    version='0.1.9',
    author='qz',
    author_email='2902934039@qq.com',
    description='方便快捷的pdf导出工具',
    long_description=open('PyPIReadme.md').read(),
    long_description_content_type="text/markdown",
    url='',
    packages=find_packages(),
    include_package_data=True,
    classifiers=[
        'Framework :: Django',
        'Programming Language :: Python :: 3',
    ],
    install_requires=[  # 依赖列表
        # 列出你的项目依赖的外部Python包，如：'requests>=2.19.1'
    ],
)
